//
//  BibElement.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/25.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class	StringInputStream, MacroList;

#ifndef _BibElementType
typedef	enum	_BibElementType {
	BibElementType_string,
	BibElementType_number,
	BibElementType_special,
	BibElementType_braketBlock,
	BibElementType_quoteBlock,
	BibElementType_mathBlock,
	BibElementType_operatorEqual,
	BibElementType_operatorSharp,
	BibElementType_operatorComma,
	BibElementType_texCommand
} _BibElementType;
#endif

@interface BibElement : NSObject <NSCoding> {

	NSString	*elementData;
	int			elementType;
}

- (id)init;
- (void)dealloc;

+ (id)bibElementWithString:(NSString*)string;
+ (id)bibElementWithNumString:(NSString*)string;
+ (id)bibElementWithQuoteString:(NSString*)string;
+ (id)bibElementEqual;
+ (id)bibElementSharp;
+ (id)bibElementComma;
+ (id)bibElementWithSpecial:(NSString*)string;

- (void)setData:(NSString*)data;
- (int)elementType;
- (void)setType:(_BibElementType)type;

- (NSString*)stringValue;
- (NSString*)stringContents;
- (NSString*)stringWithMacro:(MacroList*)macroList;
- (NSString*)stringWithMacroInTeX:(MacroList*)macroList;

- (NSMutableArray*)getElementsFromStream:(StringInputStream*)dataStream
		error:(NSString**)errorStr needsBraket:(BOOL)needsBraket;

- (NSMutableDictionary*)analyzeElements:(NSArray*)elements
		error:(NSString**)errorStr needsBibkey:(BOOL)needsBibkey;

- (NSString*)checkNesting:(NSString*)str;
- (NSArray*)checkValidity:(NSString*)inStr outString:(NSString**)outStr;

@end
