//
//  BibPreferences.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/12/03.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "BibItem.h"
#import "BibDatabase.h"
#import	"LocalDatabase.h"

@class	MacroList;

#ifndef		BibPrefs_ListViewDefsIndex
typedef	enum	BibPrefs_ListViewDefsIndex {
	BibPrefs_ListViewDefsIndex_Title = 0,
	BibPrefs_ListViewDefsIndex_Width,
	BibPrefs_ListViewDefsIndex_MinWidth,
	BibPrefs_ListViewDefsIndex_MaxWidth,
	BibPrefs_ListViewDefsIndex_Alignment,
	BibPrefs_ListViewDefsIndex_Abbrev,
	BibPrefs_ListViewDefsIndex_Sortable,
	BibPrefs_ListViewDefsIndex_Fields,
} BibPrefs_ListViewDefsIndex;

typedef	enum	BibPref_ListViewStyle {
	BibPref_ListViewStyle_plain = 0,
	BibPref_ListViewStyle_italic,
	BibPref_ListViewStyle_bold,
	BibPref_ListViewStyle_underline
} BibPref_ListViewStyle;

#define		BibPref_ListViewStyle_italic_char		'/'
#define		BibPref_ListViewStyle_bold_char			'='
#define		BibPref_ListViewStyle_underline_char	'_'

#define		BC_PrefsChangedNotification			@"iBib Preferences Changed"
#define		BC_PrefsChanged_Macro				@"Global Macros were changed"
#define		BC_PrefsChanged_ListFormat			@"List Formats were changed"

#define		BC_UndefinedBibType						@"~~ undefined ~~"

#define		BibCompanion_HelpBookName				@"BibCompanion Help"

#endif

@interface BibPreferences : NSObject {

	NSMutableDictionary		*preferences;
	MacroList				*commonMacro;
	
	BOOL					dirty;
}

- (id)init;
- (void)dealloc;
+ (id)createGlobalObject;
+ (BibPreferences*)globalPreferences;
+ (void)releaseGlobal;

+ (NSString*)bibCompanionFolder;

+ (NSString*)stringFromLineEnding:(BC_LocalLineEnding)mode;
+ (BC_LocalLineEnding)lineEndingFromString:(NSString*)lineFeed;

+ (BibPreferences*)copyPreferencesFrom:(BibPreferences*)original;
- (void)copyFromOriginal:(BibPreferences*)original;
- (void)setDirty:(BOOL)value;

- (void)setDefaults;
- (void)loadPreferences;
- (void)savePreferences;

- (BOOL)prefInitialized;

- (MacroList*)macroList;
- (void)macroListChanged;

- (NSNumber*)listViewFontExpansionObject;
- (NSNumber*)authorFontExpansionObject;

- (NSDictionary*)bibTypeDefinitions;
- (void)setBibTypeDefinitions:(NSDictionary*)value;
- (NSArray*)bibTypesList;
- (NSString*)annoteIdentifier;
- (void)setAnnoteIdentifier:(NSString*)value;

- (NSArray*)searchFieldsTitle;
- (NSArray*)searchFieldsJournal;
- (NSArray*)searchFieldsAnnote;
- (void)setSearchFieldsTitle:(NSArray*)value;
- (void)setSearchFieldsJournal:(NSArray*)value;
- (void)setSearchFieldsAnnote:(NSArray*)value;

- (NSStringEncoding)databaseEncoding;
- (void)setDatabaseEncoding:(NSStringEncoding)value;
- (BC_LocalLineEnding)databaseLineMode;
- (void)setDatabaseLineMode:(BC_LocalLineEnding)value;

- (NSArray*)activeEncodings;
- (void)setActiveEncodings:(NSArray*)value;

- (BCListFormatType)copyItemFormat;
- (void)setCopyItemFormat:(BCListFormatType)value;
- (BOOL)convertToYen;
- (void)setConvertToYen:(BOOL)value;
- (BOOL)smartAuthorPaste;
- (void)setSmartAuthorPaste:(BOOL)value;
- (BOOL)smartTitlePaste;
- (void)setSmartTitlePaste:(BOOL)value;
- (NSArray*)properNouns;
- (void)setProperNouns:(NSArray*)value;
- (NSArray*)shortCaptalWords;
- (void)setShortCaptalWords:(NSArray*)value;
- (NSArray*)longLowercaseWords;
- (void)setLongLowercaseWords:(NSArray*)value;
- (BC_AuthorNameOrder)authorListFormat;
- (void)setAuthorListFormat:(BC_AuthorNameOrder)value;
- (float)fieldFontSize;
- (void)setFieldFontSize:(float)value;
- (float)listFontSize;
- (void)setListFontSize:(float)value;


- (BC_AuthorNameOrder)authorListFormatOnCopy;
- (void)setAuthorListFormatOnCopy:(BC_AuthorNameOrder)value;
- (NSString*)authorSeparatorOnCopy;
- (void)setAuthorSeparatorOnCopy:(NSString*)value;
- (BOOL)authorAbbrevOnCopy;
- (void)setAuthorAbbrevOnCopy:(BOOL)value;
- (int)authorNonAbbrevNum;
- (void)setAuthorNonAbbrevNum:(int)value;
- (int)authorAbbrevNum;
- (void)setAuthorAbbrevNum:(int)value;
- (BC_ListPrefix)listPrefixOnCopy;
- (void)setListPrefixOnCopy:(BC_ListPrefix)value;

- (NSString*)bibkeyGenRule;
- (void)setBibkeyGenRule:(NSString*)value;

- (BOOL)openDetailInfo;
- (void)setOpenDetailInfo:(BOOL)value;
- (NSRect)windowFrame;
- (void)setWindowFrame:(NSRect)value;
- (float)dbSplitWidth;
- (void)setDBSplitWidth:(float)value;
- (float)itemSplitWidth;
- (void)setItemSplitWidth:(float)value;

- (NSDictionary*)listViewDefinitions;
- (void)setListViewDefinitions:(NSDictionary*)value;
- (void)setListColumnWidth:(NSDictionary*)value;
- (NSArray*)listColumnOrder;
- (void)setListColumnOrder:(NSArray*)value;
- (NSString*)sortColumnIdentifier;
- (void)setSortColumnIdentifier:(NSString*)value;
- (BOOL)sortAscending;
- (void)setSortAscending:(BOOL)value;

- (NSString*)lastCreatedBibType;
- (void)setLastCreatedBibType:(NSString*)value;
- (NSString*)lastEditTab;
- (void)setLastEditTab:(NSString*)value;
- (NSString*)lastInputTab;
- (void)setLastInputTab:(NSString*)value;

- (NSArray*)inputButtonDefinitions;
- (void)setInputButtonDefinitions:(NSArray*)value;
- (BOOL)autoOpenInputPanel;
- (void)setAutoOpenInputPanel:(BOOL)value;

- (BOOL)autoSave;
- (void)setAutoSave:(BOOL)value;
- (int)autoSaveInterval;
- (void)setAutoSaveInterval:(int)value;
- (int)numBackups;
- (void)setNumBackups:(int)value;

- (BOOL)quickSearchCase;
- (void)setQuickSearchCase:(BOOL)value;

- (NSString*)lastSelectedDatabase;
- (void)setLastSelectedDatabase:(NSString*)value;
- (BCDatabaseKind)lastSelectedDbKind;
- (void)setLastSelectedDbKind:(BCDatabaseKind)value;
- (NSArray*)lastSelectedAuthors;
- (void)setLastSelectedAuthors:(NSArray*)value;
- (NSArray*)lastSelectedItems;
- (void)setLastSelectedItems:(NSArray*)value;

- (NSDictionary*)dbTreeDictionary;
- (void)setDbTreeDictionary:(NSDictionary*)dbTreeDic;

- (NSArray*)suppressWarnings;
- (BOOL)suppressWarning:(BibDatabase*)db;
- (void)appendSuppressDatabase:(BibDatabase*)db;
- (void)removeSuppressDatabase:(BibDatabase*)db;


@end
