//
//  SmartLibrary.h
//  BibCompanion
//
//  Created by Hitoshi Nakahara on 04/11/18.
//  Copyright 2004 Hit-C Soft House. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import	"BibDatabase.h"

#ifndef	BC_SearchTarget

typedef enum	BC_SearchTarget {
	BC_SearchTarget_AnyField = 1,
	BC_SearchTarget_FieldName,
	BC_SearchTarget_BibType = 10,
	BC_SearchTarget_BibKey,
	BC_SearchTarget_Authors,
	BC_SearchTarget_1stAuthor,
	BC_SearchTarget_Title,
	BC_SearchTarget_Journal,
	BC_SearchTarget_Annote,
	BC_SearchTarget_Year = 20,
	BC_SearchTarget_Modified = 25,
	BC_SearchTarget_File = 30,
	BC_SearchTarget_Contents = 40
} BC_SearchTarget;

typedef enum	BC_SearchCondition {
	BC_SearchCondition_Contains = 1,
	BC_SearchCondition_ContainsWord,
	BC_SearchCondition_NotContain,
	BC_SearchCondition_NotContainWord,
	BC_SearchCondition_Begins,
	BC_SearchCondition_Ends,
	BC_SearchCondition_Equals,
	BC_SearchCondition_Later = 20,
	BC_SearchCondition_Earlier,
	BC_SearchCondition_WithinDays = 25,
	BC_SearchCondition_BeforeDays,
	BC_SearchCondition_Exists = 30,
	BC_SearchCondition_NotExists,
	BC_SearchCondition_Identical = 40
} BC_SearchCondition;

typedef enum	BC_UpdateCondition {
	BC_UpdateCondition_Automatic = 0,
	BC_UpdateCondition_AutomaticOnLaunch,
	BC_UpdateCondition_Manual
} BC_UpdateCondition;

#endif


@class	RealDatabase, SmartLibrary, BibItem;

@interface SmartSearchCondition : NSObject {
	
	BC_SearchTarget			targetFieldChoice;
	BC_SearchCondition		conditionChoice;
	NSString				*searchString;
	BOOL					caseSensitive;

	NSCharacterSet			*wordDelimiterSet;
	NSCharacterSet			*wordCharacterSet;
}

+ (SmartSearchCondition*)newCondition;
+ (SmartSearchCondition*)conditionWithDictionary:(NSDictionary*)condDic;
+ (SmartSearchCondition*)newConditionForDupSearch;

- (id)initForDupSearch;

- (NSDictionary*)coniditionDictionary;

- (BOOL)conditionMatchesItem:(BibItem*)item;
- (BOOL)stringMatchesCondition:(NSString*)string;
- (BOOL)fieldsMatchesCondition:(NSArray*)fieldList forBibItem:(BibItem*)item;
- (BOOL)string:(NSString*)target containsWord:(NSString*)word options:(unsigned)options;

- (BC_SearchTarget)targetFieldChoice;
- (void)setTargetFieldChoice:(BC_SearchTarget)choice;
- (BC_SearchCondition)conditionChoice;
- (void)setConditionChoice:(BC_SearchCondition)choice;
- (NSString*)searchString;
- (void)setSearchString:(NSString*)string;
- (BOOL)caseSensitive;
- (void)setCaseSensitive:(BOOL)choice;

- (double)minimumTimeInterval;

- (BOOL)conditionIsIdentical;

@end

/////////////////

@interface SmartLibraryProperties : DatabaseProperties {

	NSString			*targetDatabaseID;
	BOOL				matchesAllConditions;
	BC_UpdateCondition	updateCondition;

	NSMutableArray		*searchConditions;		// array of SmartSearchCondition
}

+ (SmartLibraryProperties*)propertiesWithDatabase:(BibDatabase*)lib;
+ (SmartLibraryProperties*)propertiesWithDictionary:(NSDictionary*)propertyDic database:(BibDatabase*)lib;
+ (SmartLibraryProperties*)propertiesWithProperties:(SmartLibraryProperties*)source;

- (id)initWithDatabase:(BibDatabase*)lib;
- (void)readFromDictionary:(NSDictionary*)propertyDic;
- (void)makeCopyOf:(SmartLibraryProperties*)source;
- (NSMutableDictionary*)makePropertyDictionary;

- (NSMutableArray*)searchConditions;
- (void)setSearchConditions:(NSMutableArray*)conditions;
- (BOOL)conditionMatchesItem:(BibItem*)item;
- (BOOL)databaseOfInterest:(RealDatabase*)realDB;

- (NSString*)targetDatabaseID;
- (void)setTargetDatabaseID:(NSString*)dbID;

- (BOOL)matchesAllConditions;
- (void)setMatchesAllConditions:(BOOL)value;
- (BC_UpdateCondition)updateCondition;
- (void)setUpdateCondition:(BC_UpdateCondition)value;

- (double)minimumTimeInterval;

- (BOOL)conditionIsIdentical;

@end

/////////////////

@interface SmartLibrary : BibDatabase {

	NSLock			*mergingListLock;
	NSMutableArray	*mergingDatabaseList;
	
	NSMutableDictionary	*identicalCheckList;
	
	NSTimer				*timeDependentCheck;
}

+ (SmartLibrary*)smartLibrary;
+ (SmartLibrary*)smartLibraryWithName:(NSString*)name;
+ (SmartLibrary*)smartLibraryWithDictionary:(NSDictionary*)propertyDic;
+ (SmartLibrary*)smartLibraryForDupSearch;

- (id)initWithName:(NSString*)name;
- (id)initWithDictionary:(NSDictionary*)propertyDic;
- (id)initForDupSearch;

- (void)dealloc;

- (void)openProperty;
- (void)propertiesChanged;

- (BOOL)conditionMatchesItem:(BibItem*)bibItem;

- (BOOL)bibItemChanged:(BibItem*)item;

- (void)databaseOpened:(RealDatabase*)realDB;
- (void)databaseClosed:(RealDatabase*)realDB;

- (int)numItems;

- (void)triggerForTimeDependentCondition;

- (BOOL)manualUpdateOnLaunch:(BOOL)launch;
- (BOOL)automaticUpdateOnLaunch:(BOOL)launch;

- (void)updateSmartLibrary:(DatabaseGroup*)databases;

- (BOOL)appendItemToIdenticalList:(BibItem*)item;
- (void)removeItemFromIdenticalList:(BibItem*)item;

- (BOOL)removeBibItem:(BibItem*)bibItem removeFile:(BOOL)removeFile;

@end
